--
-- (C) 2013-18 - ntop.org
--

dirs = ntop.getDirs()
package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path

require "lua_utils"
require "flow_utils"
local json = require "dkjson"

sendHTTPContentTypeHeader('text/html')
local debug = false
local debug_process = false -- Show flow processed information

interface.select("igb1")
local ifstats = interface.getStats()
-- printGETParameters(_GET)

-- Table parameters
local currentPage = _GET["currentPage"]
local perPage     = _GET["perPage"]
local sortColumn  = _GET["sortColumn"]
local sortOrder   = _GET["sortOrder"]
local host_info   = url2hostinfo(_GET)
local port        = _GET["port"]
local application = _GET["application"]
local category    = _GET["category"]
local network_id  = _GET["network"]
local vlan        = _GET["vlan"]

local deviceIP    = _GET["deviceIP"]
local inIfIdx     = _GET["inIfIdx"]
local outIfIdx    = _GET["outIfIdx"]

local asn         = _GET["asn"]

local vhost       = _GET["vhost"]
local flowhosts_type  = _GET["flowhosts_type"]
local ipversion       = _GET["version"]
local traffic_type = _GET["traffic_type"]
local flow_status = _GET["flow_status"]

-- System host parameters
local hosts  = _GET["hosts"]
local user   = _GET["username"]
local host   = _GET["host"]
local pid    = tonumber(_GET["pid"])
local name   = _GET["pid_name"]

-- Get from redis the throughput type bps or pps
local throughput_type = getThroughputType()

local prefs = ntop.getPrefs()

if(network_id ~= nil) then
   network_id = tonumber(network_id)
end

if sortColumn == nil or sortColumn == "" or sortColumn == "" then
   sortColumn = getDefaultTableSort("flows")
elseif sortColumn ~= "" and  sortColumn ~= "" then
   tablePreferences("sort_flows",sortColumn)
else
   sortColumn = "client"
end

if sortOrder == nil then
  sortOrder = getDefaultTableSortOrder("flows")
elseif sortColumn ~= "" and sortColumn ~= "" then
  tablePreferences("sort_order_flows",sortOrder)
end

if(currentPage == nil) then
   currentPage = 1
else
   currentPage = tonumber(currentPage)
end


if(port ~= nil) then port = tonumber(port) end
local flows_stats1 = interface.getFlowsInfo(host, pageinfo)
local total1 = flows_stats1["numFlows"]
perPage = total1

local to_skip = (currentPage - 1) * perPage

-- io.write("->"..sortColumn.."/"..perPage.."/"..sortOrder.."/"..sortColumn.."\n")
interface.select("igb1")
local a2z = false
if(sortOrder == "desc") then a2z = false else a2z = true end

local pageinfo = {
   ["sortColumn"] = sortColumn, ["toSkip"] = to_skip, ["maxHits"] = perPage,
   ["a2zSortOrder"] = a2z,
   ["hostFilter"] = host,
   ["portFilter"] = port,
   ["LocalNetworkFilter"] = network_id
}

if application ~= nil and application ~= "" then
   pageinfo["l7protoFilter"] = interface.getnDPIProtoId(application)
end

if category ~= nil and category ~= "" then
   pageinfo["l7categoryFilter"] = interface.getnDPICategoryId(category)
end

if not isEmptyString(flowhosts_type) then
   if flowhosts_type == "local_origin_remote_target" then
      pageinfo["clientMode"] = "local"
      pageinfo["serverMode"] = "remote"
   elseif flowhosts_type == "local_only" then
      pageinfo["clientMode"] = "local"
      pageinfo["serverMode"] = "local"
   elseif flowhosts_type == "remote_origin_local_target" then
      pageinfo["clientMode"] = "remote"
      pageinfo["serverMode"] = "local"
   elseif flowhosts_type == "remote_only" then
      pageinfo["clientMode"] = "remote"
      pageinfo["serverMode"] = "remote"
   end
end

if not isEmptyString(traffic_type) then
   if traffic_type:contains("unicast") then
      pageinfo["unicast"] = true
   else
      pageinfo["unicast"] = false
   end

   if traffic_type:contains("one_way") then
      pageinfo["unidirectional"] = true
   end
end

if not isEmptyString(flow_status) then
   if flow_status == "normal" then
      pageinfo["alertedFlows"] = false
      pageinfo["filteredFlows"] = false
   elseif flow_status == "alerted" then
      pageinfo["alertedFlows"] = true
   elseif flow_status == "filtered" then
      pageinfo["filteredFlows"] = true
   end
end

if not isEmptyString(ipversion) then
   pageinfo["ipVersion"] = tonumber(ipversion)
end

if not isEmptyString(vlan) then
   pageinfo["vlanIdFilter"] = tonumber(vlan)
end

if not isEmptyString(deviceIP) then
   pageinfo["deviceIpFilter"] = deviceIP

   if not isEmptyString(inIfIdx) then
      pageinfo["inIndexFilter"] = tonumber(inIfIdx)
   end

   if not isEmptyString(outIfIdx) then
      pageinfo["outIndexFilter"] = tonumber(outIfIdx)
   end
end

if not isEmptyString(asn) then
   pageinfo["asnFilter"] = tonumber(asn)
end

local flows_stats = interface.getFlowsInfo(host, pageinfo)
local total = flows_stats["numFlows"]
local flows_stats = flows_stats["flows"]

-- Prepare host
local host_list = {}
local num_host_list = 0
local single_host = 0

if(hosts ~= nil) then host_list, num_host_list = getHostCommaSeparatedList(hosts) end
if(host ~= nil) then
   single_host = 1
   num_host_list = 1
end

if(flows_stats == nil) then flows_stats = { } end

for key, value in ipairs(flows_stats) do
   local info = ""
   if(not isEmptyString(flows_stats[key]["info"])) then
      info = shortenString(flows_stats[key]["info"])
   elseif(not isEmptyString(flows_stats[key]["icmp"])) then
      info = getICMPTypeCode(flows_stats[key]["icmp"])
   elseif(flows_stats[key]["proto.ndpi"] == "SIP") then
      info = getSIPInfo(flows_stats[key])
   elseif(flows_stats[key]["proto.ndpi"] == "RTP") then
      info = getRTPInfo(flows_stats[key])
   end
   flows_stats[key]["info"] = info

   if(flows_stats[key]["profile"] ~= nil) then
      flows_stats[key]["info"] = flows_stats[key]["profile"]..info
   end
end

local formatted_res = {}

for _key, value in ipairs(flows_stats) do -- pairsByValues(vals, funct) do
   local record = {}
   local key = value["ntopng.key"]

   local srv_name = flowinfo2hostname(value, "srv")
   local cli_name = flowinfo2hostname(value, "cli")
   local find_bytes = round(value["bytes"]/1024/1024, 2)
   if find_bytes < 1 then
	break
	return false
   end
   
   

   if(cli_name == nil) then cli_name = "???" end
   if(srv_name == nil) then srv_name = "???" end

   local cli_tooltip =value["tcp.nw_latency.client"]
   local srv_tooltip =value["tcp.nw_latency.server"]

 
   
	if(value["tcp.nw_latency.client"] ~= nil) then
      local rtt = value["tcp.nw_latency.client"] + value["tcp.nw_latency.server"]

      if(rtt > 0) then
	 local cli2srv = round(((value["tcp.nw_latency.client"] * 100) / rtt), 2)
	 local srv2cli = round(((value["tcp.nw_latency.server"] * 100) / rtt), 2)	 
	 
	 cli_tooltip=round(value["tcp.nw_latency.client"],2)
	 srv_tooltip=round(value["tcp.nw_latency.server"],2)
		end
	end
	
	
	record["cli_tooltip"] = cli_tooltip

	record["srv_tooltip"] = srv_tooltip
   
	src_key = shortenString(stripVlan(cli_name))


	dst_key = shortenString(stripVlan(srv_name))

	
	record["client_ip"] = value["cli.ip"]

	record["server_ip"] = value["srv.ip"]
	
	
	record["client_port"] = ""..value["cli.port"]
	record["server_port"] = ""..value["srv.port"]
	
	local trend_state = value["throughput_trend_bps"]
	if trend_state == 1 then
      trend_state="Upload"
	 elseif trend_state == 2 then
	 trend_state="Download"
	 elseif trend_state == 3 then
	 trend_state="Hold"
	end
	
	record["throughput_trend_bps"] = trend_state


	
	
	
	
   record["key"] = value["ntopng.key"]

   record["key"] = value["ntopng.key"]

   local column_client = src_key
   local info = interface.getHostInfo(value["cli.ip"], value["cli.vlan"])

   if(info ~= nil) then
      column_client = column_client..getFlag(info["country"])
   end

   column_client_ip = column_client
   record["client_name"] = column_client_ip

   local column_server = dst_key
   info = interface.getHostInfo(value["srv.ip"], value["srv.vlan"])
   if(info ~= nil) then
      column_server = column_server..getFlag(info["country"])
   end

   column_server_ip = column_server
   record["server_name"] = column_server_ip
   

   
   record["cli2srv_bytes"] = bytesToSize(value["cli2srv.bytes"])
   record["srv2cli_bytes"] = bytesToSize(value["srv2cli.bytes"])
   record["cli2srv_bytes_full"] = value["cli2srv.bytes"]
   record["srv2cli_bytes_full"] = value["srv2cli.bytes"]

   record["vlan"] = ''
   if((value["vlan"] ~= nil)) then
      record["vlan"] = value["vlan"]
   end

   local column_proto_l4 = ''

   if(interface.isPacketInterface()) then
      if(value["flow.status"] ~= 0) then
	 column_proto_l4 = string.gsub(getFlowStatus(value["flow.status"]), "<[^>]*>([^<]+)<.*", "%1")
      end
   end
   record["flow_status_string"] = string.gsub(getFlowStatus(value["flow.status"]), "<[^>]*>([^<]+)<.*", "%1")


   if value["proto.l4"] == "TCP" or value["proto.l4"] == "UDP" then
      if value["tcp.seq_problems"] == true then
	 local tcp_issues = ""
	 if value["cli2srv.out_of_order"] > 0 or value["srv2cli.out_of_order"] > 0 then
	    tcp_issues = tcp_issues.." Out-of-order"
	 end
	 if value["cli2srv.retransmissions"] > 0 or value["srv2cli.retransmissions"] > 0 then
	    tcp_issues = tcp_issues.." Retransmissions"
	 end
	 if value["cli2srv.lost"] > 0 or value["srv2cli.lost"] > 0 then
	    tcp_issues = tcp_issues.." Loss"
	 end

	 column_proto_l4 = column_proto_l4..'Issues detected:'..tcp_issues..' '..value["proto.l4"]
      elseif value["flow_goodput.low"] == true then
	 column_proto_l4 = column_proto_l4..'Low Goodput '..value["proto.l4"]
      else
	 column_proto_l4 = column_proto_l4..' '..value["proto.l4"]
      end
   else
      column_proto_l4 = column_proto_l4..value["proto.l4"]
   end
   record["proto_l4"] = column_proto_l4


   if(value["verdict.pass"] == false) then
      verdict_pass = value["verdict.pass"]
	 else
		verdict_pass="true"
   end
   record["verdict_pass"] = verdict_pass
   record["ndpi_id"] = value["proto.ndpi_id"]
   record["ndpi_name"] = value["proto.ndpi"]
   record["duration"] = secondsToTime(value["duration"])
   record["bytes"] = bytesToSize(value["bytes"])
   record["bytes_new"] = value["bytes"]

   record["i18n_details_first_last_seen"] = i18n("details.first_last_seen")
   record["formatEpoch_seen_first"] = formatEpoch(value["seen.first"])
   record["seen_first"] = value["seen.first"]
   record["secondsToTime_seen_first"] = secondsToTime(os.time()-value["seen.first"])
   record["details_ago"] = i18n("details.ago")
   record["formatEpoch_seen_last"] = formatEpoch(value["seen.last"])
   record["seen_last"] = value["seen.last"]
   record["secondsToTime_seen_last"] = secondsToTime(os.time()-value["seen.last"])



--i18n("details.first_last_seen")
--formatEpoch(flow["seen.first"])
--secondsToTime(os.time()-flow["seen.first"])
--i18n("details.ago") [01:13:22 ago]

--formatEpoch(flow["seen.last"])
--secondsToTime(os.time()-flow["seen.last"])
--i18n("details.ago") [00:30 ago]




   local column_thpt = ''
   if(throughput_type == "pps") then
      column_thpt = column_thpt..pktsToSize(value["throughput_pps"])
   else
      column_thpt = column_thpt..bitsToSize(8*value["throughput_bps"])
   end

   record["thpt"] = column_thpt

   local cli2srv = round((value["cli2srv.bytes"] * 100) / value["bytes"], 0)

   record["breakdown_client"] = cli2srv
   record["breakdown_server"] = 100-cli2srv

   local info = value["info"]

   record["info"] = info

   formatted_res[#formatted_res + 1] = record

end -- for

if(sortColumn == nil) then
   sortColumn = ""
end

if(sortOrder == nil) then
   sortOrder = ""
end

local result = {}

result["perPage"] = perPage
result["currentPage"] = currentPage
result["totalRows"] = total
result["data"] = formatted_res
result["sort"] = {{sortColumn, sortOrder}}

print(json.encode(result))
